/**
 * Copyright (C) 2020-2023 DCsunset
 * See full notice in README.md in this project
 */
import { DateTime } from "luxon";
import { taskUrgency, dueUrgency } from "task.json";
import { isTaskStatus } from "task.json/dist/index.guard.js";
const statusMap = {
    t: "todo",
    d: "done",
    r: "removed"
};
export function numbersToIndexes(nums) {
    const handleError = (num) => {
        throw new Error(`Invalid Number: ${num}`);
    };
    // Keep only unique nums
    return [...new Set(nums)].map(num => {
        if (num.length <= 1)
            handleError(num);
        const s = num.substring(0, 1);
        const n = parseInt(num.substring(1));
        if (isNaN(n) || n < 0 || !(s in statusMap))
            handleError(num);
        return [statusMap[s], n];
    });
}
export function colorTask(task) {
    const urgency = taskUrgency(task);
    // gray if it's pending or has todo dependant
    if (!filterByWait(false)(task) || !filterByDeps(false)(task))
        return "gray";
    if (urgency >= 1000)
        return "red";
    if (urgency >= 600)
        return "yellow";
    if (urgency >= 2)
        return "cyan";
    return null;
}
export function colorPriority(pri) {
    switch (pri) {
        case "A":
            return "red";
        case "B":
            return "yellow";
        default:
            return "cyan";
    }
}
export function colorDue(due) {
    const urgency = dueUrgency(due);
    if (urgency >= 1000)
        return "red";
    if (urgency >= 600)
        return "yellow";
    return null;
}
export function idsToNumbers(classified, ids) {
    return ids.map(id => {
        let num = null;
        for (const [status, tasks] of Object.entries(classified)) {
            const index = tasks.findIndex(t => t.id === id);
            if (index !== -1) {
                num = `${status.charAt(0)}${index}`;
                break;
            }
        }
        if (!num) {
            throw new Error(`Invalid id: ${id}`);
        }
        return num;
    });
}
export function indexesToTasks(classified, indexes) {
    return indexes.map(([status, i]) => {
        if (i >= classified[status].length)
            throw new Error(`Invalid index: ${status.substring(0, 1)}${i}`);
        return classified[status][i];
    });
}
/// Find tasks for given numbers
export function numbersToTasks(classified, numbers) {
    const indexes = numbersToIndexes(numbers);
    return indexesToTasks(classified, indexes);
}
export function numbersToIds(classified, numbers) {
    return numbersToTasks(classified, numbers).map(t => t.id);
}
export function indexesToIds(classified, indexes) {
    return indexesToTasks(classified, indexes).map(t => t.id);
}
export function normalizeStatuses(statuses) {
    const result = new Set();
    for (const st of statuses) {
        if (st === "all") {
            result.add("todo");
            result.add("done");
            result.add("removed");
        }
        else {
            if (!isTaskStatus(st)) {
                throw new Error(`Invalid status: ${st}`);
            }
            result.add(st);
        }
    }
    return [...result];
}
export function filterByWait(flag) {
    // Showing all tasks if flag is true
    if (flag)
        return () => true;
    return (task) => {
        if (task.wait) {
            const date = DateTime.fromISO(task.wait);
            // true if the wait date is after current date
            return DateTime.now() >= date;
        }
        return true;
    };
}
export function filterByDeps(flag) {
    // Showing all tasks if flag is true
    if (flag)
        return () => true;
    return (task) => {
        if (task.deps)
            for (const dep of task.deps) {
                // Deps in todo
                if (dep.startsWith("t"))
                    return false;
            }
        return true;
    };
}
export function filterByPriority(priorities) {
    if (!priorities)
        return () => true;
    return (task) => {
        for (const p of priorities) {
            if (p.length === 0 && task.priority === undefined)
                return true;
            if (p.length > 0 && task.priority === p)
                return true;
        }
        return false;
    };
}
// Filter by projects or contexts
export function filterByField(field, values) {
    if (!values)
        return () => true;
    return (task) => {
        // whether there's positive filter
        let hasPos = false;
        let posResult = false;
        let negResult = true;
        values.forEach(value => {
            if (value.length === 0)
                posResult = posResult || task[field] === undefined;
            else if (value[0] === "!") {
                // use and operator for negative filter
                negResult = negResult && !(task[field]?.includes(value.substring(1)));
            }
            else {
                posResult = posResult || Boolean(task[field]?.includes(value));
                hasPos = true;
            }
        });
        return negResult && (!hasPos || posResult);
    };
}
export function stringifyDiffStat(stat) {
    return Object.entries(stat)
        .map(([key, value]) => `${key}: ${value}`)
        .join(", ");
}
export function maxWidth(tasks, deps, wait) {
    const initWidths = {
        text: 4,
        contexts: 3,
        projects: 4,
        deps: deps ? 3 : 0,
        wait: wait ? 4 : 0,
        due: 3
    };
    return tasks.reduce((widths, task) => {
        for (const [field, value] of Object.entries(widths)) {
            if (field === "deps" && !deps)
                continue;
            if (field === "wait" && !wait)
                continue;
            widths[field] = Math.max(task[field].length, value);
        }
        return widths;
    }, initWidths);
}
//# sourceMappingURL=task.js.map